@@ -34,8 +34,6 @@ def KDEuse(lenv, flags):
 		lenv.KDEaddflags_cxx( ['-DQT_THREAD_SUPPORT', '-D_REENTRANT'] )
 	if 'fastmoc' in _flags:
 		lenv['BKSYS_FASTMOC']=1
-	if 'dump' in _flags:
-		lenv['DUMPCONFIG']=1
 	if not 'nohelp' in _flags:
 		if lenv['_CONFIGURE'] or lenv['HELP']: lenv.Exit(0)
 	if not 'nosmart' or not lenv.has_key('nosmart_includes'):
@@ -285,20 +283,10 @@ def generate(env):
 	import SCons.Util
 	import SCons.Node
 
-	def reldir(dir):
-		ndir    = SCons.Node.FS.default_fs.Dir(dir).srcnode().abspath
-		rootdir = SCons.Node.FS.default_fs.Dir('#').abspath
-		return ndir.replace(rootdir, '').lstrip('/')
-
-	def relfile(file):
-		nfile   = SCons.Node.FS.default_fs.File(file).srcnode().abspath
-		rootdir = SCons.Node.FS.default_fs.Dir('#').abspath
-		return nfile.replace(rootdir, '').lstrip('/')
-
 	CLVar = SCons.Util.CLVar
 	splitext = SCons.Util.splitext
 	Builder = SCons.Builder.Builder
-
+	
 	# Detect the environment - replaces ./configure implicitely and store the options into a cache
 	from SCons.Options import Options
 	cachefile=env['CACHEDIR']+'kde.cache.py'
@@ -347,7 +335,7 @@ def generate(env):
 	## set default variables, one can override them in sconscript files
 	env.Append(CXXFLAGS = ['-I'+env['KDEINCLUDEPATH'], '-I'+env['QTINCLUDEPATH'] ],
 			LIBPATH = [env['KDELIBPATH'], env['QTLIBPATH'] ])
-
+	
 	env['QT_AUTOSCAN'] = 1
 	env['QT_DEBUG']    = 0
 
@@ -392,8 +380,9 @@ def generate(env):
 		bs = env.join(str(target[0].get_dir()),bs)
 		# .h file is already there
 		target.append( bs+'.cpp' )
-
+		
 		content=source[0].srcnode().get_contents()
+		#print content
 
 		kcfgfilename=""
 		kcfgFileDeclRx = re.compile("[fF]ile\s*=\s*(.+)\s*")
@@ -408,7 +397,7 @@ def generate(env):
 
 	env['BUILDERS']['Kcfg']=Builder(action=env.Action(kcfg_buildit, kcfg_stringit),
 			emitter=kcfgEmitter, suffix='.h', src_suffix='.kcfgc')
-
+	
 	## MOC processing
 	env['BUILDERS']['Moc']=Builder(action='$QT_MOC -o $TARGET $SOURCE',suffix='.moc',src_suffix='.h')
 	env['BUILDERS']['Moccpp']=Builder(action='$QT_MOC -o $TARGET $SOURCE',suffix='_moc.cpp',src_suffix='.h')
@@ -423,8 +412,7 @@ def generate(env):
 	env['BUILDERS']['Stub']=Builder(action= 'dcopidl2cpp --c++-suffix cpp --no-signals --no-skel $SOURCE',
 			suffix='_stub.cpp', src_suffix='.kidl')
 	## DOCUMENTATION
-	env['BUILDERS']['Meinproc']=Builder(action='cd $TARGET.dir && $MEINPROC --check --cache $TARGET.name $SOURCE.name',
-			suffix='.cache.bz2')
+	env['BUILDERS']['Meinproc']=Builder(action='cd $TARGET.dir && $MEINPROC --check --cache $TARGET.name $SOURCE.name',suffix='.cache.bz2')
 	## TRANSLATIONS
 	env['BUILDERS']['Transfiles']=Builder(action='$MSGFMT $SOURCE -o $TARGET',suffix='.gmo',src_suffix='.po')
 
@@ -439,13 +427,10 @@ def generate(env):
 	stub_ext = [".stub", ".STUB"]
 
 	def KDEfiles(lenv, target, source):
-		""" Returns a list of files for scons (handles kde tricks like .skel)
+		""" Returns a list of files for scons (handles kde tricks like .skel) 
 		It also makes custom checks against double includes like : ['file.ui', 'file.cpp']
 		(file.cpp is already included because of file.ui) """
 
-		# ITA
-		#print "kdefiles"
-
 		q_object_search = re.compile(r'[^A-Za-z0-9]Q_OBJECT[^A-Za-z0-9]')
 		def scan_moc(cppfile):
 			addfile=None
@@ -489,7 +474,6 @@ def generate(env):
 
 		# For each file, check wether it is a dcop file or not, and create the complete list of sources
 		for file in source_:
-
 			sfile=SCons.Node.FS.default_fs.File(str(file)) # why str(file) ? because ordinal not in range issues
 			bs  = SCons.Util.splitext(file)[0]
 			ext = SCons.Util.splitext(file)[1]
@@ -510,7 +494,7 @@ def generate(env):
 				src.append(file)
 				if not env.has_key('NOMOCFILE'):
 					ret = scan_moc(sfile)
-					if ret: src.append( sfile.dir.File(ret) )
+					if ret: src.append( ret )
 			elif ext in ui_ext:
 				lenv.Uic(file)
 				src.append(bs+'.cpp')
@@ -523,7 +507,7 @@ def generate(env):
 				src.append(file)
 
 		for base in kidl: lenv.Kidl(base+'.h')
-
+		
 		# Now check against typical newbie errors
 		for file in ui_files:
 			for ofile in other_files:
@@ -535,8 +519,6 @@ def generate(env):
 				if ofile == file:
 					env.pprint('RED',"WARNING: You have included %s.kcfg and another file of the same prefix"%file)
 					print "Files generated by kconfig_compiler (settings.h, settings.cpp) must not be included"
-		# ITA
-		#print "end kdefiles"
 		return src
 
 
@@ -544,12 +526,10 @@ def generate(env):
 	configuration for re-use from an IDE """
 	def KDEinstall(lenv, restype, subdir, files, perms=None):
 		if env.has_key('DUMPCONFIG'):
-			ret= "<install type=\"%s\" subdir=\"%s\">\n" % (restype, subdir)
-			for i in lenv.make_list(files):
-				ret += "    <file name=\"%s\"/>\n" % (relfile(i))
-			ret += "</install>\n"
-			lenv.add_dump(ret)
-			return None
+			print "<install type=\"%s\" subdir=\"%s\">" % (restype, subdir)
+			for i in lenv.make_list(files): print "    <file name=\"%s\"/>" % (lenv.join(lenv.getreldir(),i))
+			print "</install>"
+			return
 
 		if not env['_INSTALL']: return None
 		dir = getInstDirForResType(lenv, restype)
@@ -568,9 +548,9 @@ def generate(env):
                 env.Alias('install', install_list)
 		return install_list
 
-	def KDEprogram(lenv, target, source,
+	def KDEprogram(lenv, target, source, 
 			includes='', localshlibs='', globallibs='', globalcxxflags=''):
-		""" Makes a kde program
+		""" Makes a kde program 
 		The program is installed except if one sets env['NOAUTOINSTALL'] """
 		src = KDEfiles(lenv, target, source)
 		program_list = lenv.Program(target, src)
@@ -582,16 +562,16 @@ def generate(env):
 			lst=lenv.make_list(localshlibs)
 			lenv.link_local_shlib(lst)
 			lenv.Depends( program_list, lst )
-
+			
 		if len(includes)>0:       lenv.KDEaddpaths_includes(includes)
 		if len(globallibs)>0:     lenv.KDEaddlibs(globallibs)
 		if len(globalcxxflags)>0: lenv.KDEaddflags_cxx(globalcxxflags)
-
+		
 		if not lenv.has_key('NOAUTOINSTALL'):
 			KDEinstall(lenv, 'KDEBIN', '', target)
 		return program_list
 
-	def KDEshlib(lenv, target, source, kdelib=0, libprefix='lib',
+	def KDEshlib(lenv, target, source, kdelib=0, libprefix='lib', 
 			includes='', localshlibs='', globallibs='', globalcxxflags='', vnum=''):
 		""" Makes a shared library for kde (.la file for klibloader)
 		The library is installed except if one sets env['NOAUTOINSTALL'] """
@@ -617,7 +597,7 @@ def generate(env):
 		return library_list
 
 	def KDEstaticlib(lenv, target, source):
-		""" Makes a static library for kde - in practice you should not use static libraries
+		""" Makes a static library for kde - in practice you should not use static libraries 
 		1. they take more memory than shared ones
 		2. makefile.am needed it because of limitations
 		(cannot handle sources in separate folders - takes extra processing) """
@@ -629,7 +609,7 @@ def generate(env):
 	def KDEaddflags_cxx(lenv, fl):
 		""" Compilation flags for C++ programs """
 		lenv.AppendUnique(CXXFLAGS = lenv.make_list(fl))
-
+	
 	def KDEaddflags_c(lenv, fl):
 		""" Compilation flags for C programs """
 		lenv.AppendUnique(CFLAGS = lenv.make_list(fl))
@@ -657,33 +637,19 @@ def generate(env):
 		fld=dir.srcnode()
 		tmptransfiles = glob.glob(str(fld)+'/*.po')
 
-		if lenv.has_key('DUMPCONFIG'):
-			lenv.add_dump( "<podir dir=\"%s\" name=\"%s\"/>\n" % (reldir(dir), appname) )
-			return
-
 		transfiles=[]
 		if lenv.has_key('_BUILDDIR_'):
 			bdir=lenv['_BUILDDIR_']
-			for pof in lenv.make_list(tmptransfiles):
-				# ITA
-				d=relfile(pof)
-				d2=d.replace(bdir,'').lstrip('/')
-				d3=lenv.join(bdir, d2)
-				d4=d3[:d3.rfind('/')]
-				#print d2
-				#print d3
-				#print d4
-				transfiles.append( lenv.join('#', bdir, d2) )
-				os.popen('mkdir -p ' + d4 + ' && ln -f ' + d2 + ' ' + d3)
-		else: transfiles=tmptransfiles
+			for dir in lenv.make_list(tmptransfiles):
+				transfiles.append( lenv.join(bdir, dir) )
+		else: tmptransfiles=transfiles
 
 		languages=None
 		if lenv['ARGS'] and lenv['ARGS'].has_key('languages'):
 			languages=lenv.make_list(lenv['ARGS']['languages'])
 		mydir=SCons.Node.FS.default_fs.Dir('.')
 		for f in transfiles:
-			#fname=f.replace(mydir.abspath, '')
-			fname=f
+			fname=f.replace(mydir.abspath, '')
 			file=SCons.Node.FS.default_fs.File(fname)
 			country = SCons.Util.splitext(file.name)[0]
 			if not languages or country in languages:
@@ -695,9 +661,9 @@ def generate(env):
 		"""Contributed by: "Andrey Golovizin" <grooz()gorodok()net>
 		modified by "Martin Ellis" <m.a.ellis()ncl()ac()uk>
 
-		Installs icons with filenames such as cr22-action-frame.png into
+		Installs icons with filenames such as cr22-action-frame.png into 
 		KDE icon hierachy with names like icons/crystalsvg/22x22/actions/frame.png.
-
+		
 		Global KDE icons can be installed simply using env.KDEicon('name').
 		The second parameter, path, is optional, and specifies the icons
 		location in the source, relative to the SConscript file.
@@ -706,14 +672,12 @@ def generate(env):
 		avoid name conflicts, for example), use e.g.
 		env.KDEicon('name', './', 'KDEDATA', 'appname/icons')"""
 
-		if lenv.has_key('DUMPCONFIG'):
-			lenv.add_dump( "<icondir>\n" )
-			lenv.add_dump( "    <icondirent dir=\"%s\" subdir=\"%s\"/>\n" % (reldir(path), subdir) )
-			lenv.add_dump( "</icondir>\n" )
+		if env.has_key('DUMPCONFIG'):
+			print "<icondirent subdir=\"%s\" />" % (lenv.join(lenv.getreldir(),path,subdir))
 			return
 
 		type_dic = { 'action':'actions', 'app':'apps', 'device':'devices',
-			'filesys':'filesystems', 'mime':'mimetypes' }
+			'filesys':'filesystems', 'mime':'mimetypes' } 
 		dir_dic = {
 		'los'  :'locolor/16x16', 'lom'  :'locolor/32x32',
 		'him'  :'hicolor/32x32', 'hil'  :'hicolor/48x48',
@@ -754,7 +718,7 @@ def generate(env):
 	def docfolder(lenv, folder, lang, destination=""):
 		# folder is the folder to process
 		# lang is the language
-		# destination is the subdirectory in KDEDOC (appname)
+		# destination is the subdirectory in KDEDOC
 		import glob
 		docfiles=[]
 		dir=SCons.Node.FS.default_fs.Dir(folder).srcnode()
@@ -764,13 +728,7 @@ def generate(env):
 		for file in docg:
 			f = file.replace(dirpath, '')
 			docfiles.append(f)
-
-		if lenv.has_key('DUMPCONFIG'):
-			lenv.add_dump( "<docdir name=\"%s\">\n" % destination)
-			lenv.add_dump( "    <docdirent lang=\"%s\" dir=\"%s\"/>\n" % (lang, reldir(dir)) )
-			lenv.add_dump( "</docdir>\n" )
-			return
-
+		
 		# warn about errors
 		#if len(lang) != 2:
 		#       print "error, lang must be a two-letter string, like 'en'"
@@ -785,14 +743,8 @@ def generate(env):
 			# do not process folders
 			#if not os.path.isfile( lenv.join('.', file) ): continue
 
-			# build a node representing the file in the build directory to force symlinking
-			nodefile1=relfile( lenv.join(mydir.abspath, file) )
-			#print nodefile1
-			#nodefile=relfile( lenv.join(self.dirprefix, file) )
-			nodefile=SCons.Node.FS.default_fs.File( lenv.join('#', bdir, nodefile1) )
-			#print nodefile.abspath
-			d4 = nodefile.abspath[:nodefile.abspath.rfind('/')]
-			os.popen('mkdir -p ' + d4 + ' && ln -f ' + nodefile1 + ' ' + nodefile.abspath)
+			# using nodefile forces to symlink in builddir
+			nodefile=SCons.Node.FS.default_fs.File( lenv.join(mydir.abspath, file) )
 
 			# do not process the cache file
 			if file == 'index.cache.bz2': continue
@@ -801,42 +753,34 @@ def generate(env):
 			ext = SCons.Util.splitext( file )[1]
 
 			# install picture files
-			if ext in ['.jpeg', '.jpg', '.png']: lenv.KDEinstall('KDEDOC', lenv.join(lang,destination), nodefile.abspath)
+			if ext in ['.jpeg', '.jpg', '.png']: lenv.KDEinstall('KDEDOC', lenv.join(lang,destination), nodefile.srcnode())
                         # docbook files are processed by meinproc
 			if ext != '.docbook': continue
-
+			
 			docbook_list.append( nodefile )
-			lenv.KDEinstall('KDEDOC', lenv.join(lang,destination), nodefile.abspath)
+			lenv.KDEinstall('KDEDOC', lenv.join(lang,destination), nodefile.srcnode())
 
 		# Now process the index.docbook files ..
 		if len(docbook_list) == 0: return
-
-		index=''
-		cache=''
+		# TODO
+		#if not os.path.isfile( folder+'/index.docbook' ):
+		#	print "Error, index.docbook was not found in "+folder+'/index.docbook'
+		#	return
+		## Define this to 1 if you are writing documentation else to 0 :)
+		#if lenv.has_key('i_am_a_documentation_writer'):
 		for file in docbook_list:
-			if file.name=='index.docbook':
-				index=file
-				cache=file.dir.File('index.cache.bz2')
+			lenv.Depends( folder+'index.cache.bz2', nodefile )
 
-		if not index:
-			print "BUG in docfolder: no index.docbook but docbook files found"
-			lenv.Exit(1)
-
-		for file in docbook_list:
-			# make the cache.bz2 file depend on all .docbook files in the same dir
-			lenv.Depends( cache, file )
-
-		lenv.Meinproc( cache, index )
-		lenv.KDEinstall( 'KDEDOC', lenv.join(lang,destination), cache )
+		if lenv.has_key('_BUILDDIR_'): folder=lenv.join(lenv['_BUILDDIR_'], folder)
 
+		lenv.Meinproc( lenv.join(folder,'index.cache.bz2'), lenv.join(folder,'index.docbook') )
+		lenv.KDEinstall( 'KDEDOC', lenv.join(lang,destination), lenv.join(folder,'index.cache.bz2') )
+		
 		if env['_INSTALL']:
 			dir=lenv.join(lenv.getInstDirForResType('KDEDOC'), lang, destination)
 			comp='mkdir -p %s && cd %s && rm -f common && ln -s ../common common' % (dir, dir)
 			lenv.Execute(comp)
 
-			#self.env.AddPostAction(lenv.join(dir, 'common'), self.env.Chmod(ins, self.perms))
-			# TODO add post action of cache (index.cache.bz2)
-
 	#valid_targets = "program shlib kioslave staticlib".split()
 	import generic
 	class kobject(generic.genobj):
@@ -847,9 +791,11 @@ def generate(env):
 		def it_is_a_kdelib(self): self.iskdelib=1
 		def execute(self):
 			if self.executed: return
+			self.lockchdir()
 			if self.orenv.has_key('DUMPCONFIG'):
+				print self.xml()
+				self.unlockchdir()
 				self.executed=1
-				self.xml()
 				return
 			if (self.type=='shlib' or self.type=='kioslave'):
 				install_dir = 'KDEMODULE'
@@ -859,22 +805,17 @@ def generate(env):
 				self.instdir=getInstDirForResType(self.orenv, 'KDEBIN')
 				self.perms=0755
 
-			# ITA
-			#print self.source
-			#print "hallo"
 			self.p_localsource=KDEfiles(env, self.joinpath(self.target), self.joinpath(self.source))
-			# ITA
-			#print self.p_localsource
 			generic.genobj.execute(self)
+			self.unlockchdir()
 
 		def xml(self):
-			dirprefix = reldir('.')
-			if not dirprefix: dirprefix=self.dirprefix
-			ret='<compile type="%s" dirprefix="%s" target="%s" cxxflags="%s" cflags="%s" includes="%s" linkflags="%s" libpaths="%s" libs="%s" vnum="%s" iskdelib="%s" libprefix="%s">\n' % (self.type, dirprefix, self.target, self.cxxflags, self.cflags, self.includes, self.linkflags, self.libpaths, self.libs, self.vnum, self.iskdelib, self.libprefix)
+			chdirto=self.orenv.join(self.orenv.getreldir(),self.chdir)
+			ret= '<compile type="%s" chdir="%s" target="%s" cxxflags="%s" cflags="%s" includes="%s" linkflags="%s" libpaths="%s" libs="%s" vnum="%s" iskdelib="%s" libprefix="%s">\n' % (self.type, chdirto, self.target, self.cxxflags, self.cflags, self.includes, self.linkflags, self.libpaths, self.libs, self.vnum, self.iskdelib, self.libprefix)
 			if self.source:
 				for i in self.orenv.make_list(self.source): ret+='  <source file="%s"/>\n' % i
-			ret+="</compile>\n"
-			self.orenv.add_dump(ret)
+			ret += "</compile>"
+			return ret
 
 	# Attach the functions to the environment so that SConscripts can use them
 	SConsEnvironment.KDEprogram = KDEprogram
