@@ -205,7 +205,7 @@ static const UDSEntry pathToUDSEntry(const QString& path, const QString& display
 
     if (!path.isEmpty()) {
         struct stat info;
-        lstat(path, &info);
+        lstat(path.local8Bit(), &info);
 
         addAtom(entry, KIO::UDS_SIZE, info.st_size);
         addAtom(entry, KIO::UDS_ACCESS, info.st_mode);
